@@ -1,7 +1,7 @@
 /* note.c
 
-Copyright 2019-2021 Daniel T. Borelli <danieltborelli@gmail.com>
-Copyright 2021      Guilherme Janczak <guilherme.janczak@yandex.com>
+Copyright 2019-2022 Daniel T. Borelli <danieltborelli@gmail.com>
+Copyright 2021-2023 Guilherme Janczak <guilherme.janczak@yandex.com>
 Copyright 2021      IFo Hancroft <contact@ifohancroft.com>
 Copyright 2021      Peter Wu <peterwu@hotmail.com>
 
@@ -26,12 +26,17 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 */
 
-/* This file is part of the scrot project. */
-
 #include <assert.h>
-#include <stdint.h>
+#include <err.h>
+#include <limits.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
 
-#include "scrot.h"
+#include <Imlib2.h>
+
+#include "note.h"
+#include "options.h"
 
 enum { // default color
     DEF_COLOR_RED = 0,
@@ -40,93 +45,120 @@ enum { // default color
     DEF_COLOR_ALPHA = 255
 };
 
-ScrotNote note;
+/*
+ * Format: -f 'NAME/SIZE' -x NUM -y NUM -t 'TEXT' -c NUM,NUM,NUM,NUM
+ *
+ * -f fontname/size - absolute path
+ * -x screen position x
+ * -y screen position y
+ * -t text note
+ * -c color(red,green,blue,alpha) range 0..255
+ *
+ * */
 
-static Imlib_Font imFont = NULL;
+struct ScrotNote {
+    char *font;         /* font name                   */
+    char *text;         /* text of the note            */
+    Imlib_Font imFont;  /* private                     */
+    int x;              /* position screen (optional)  */
+    int y;              /* position screen (optional)  */
+    double angle;       /* angle text (optional)       */
+    struct Color {      /* (optional)                  */
+        int r,          /* red                         */
+            g,          /* green                       */
+            b,          /* blue                        */
+            a;          /* alpha                       */
+    } color;
+};
 
+static struct ScrotNote *note;
 static void loadFont(void);
+static char *parseText(char **, char const *const);
 
-static char* parseText(char**, char* const);
-
-static inline void pfree(char** ptr)
+static void pfree(char **ptr)
 {
     free(*ptr);
     *ptr = NULL;
 }
 
-static inline void nextSpace(char** token)
+static void nextSpace(char **token)
 {
     while (*++*token == ' ' && **token != '\0') { }
 }
 
-static inline void nextNotSpace(char** token)
+static void nextNotSpace(char **token)
 {
     while (*++*token != ' ' && **token != '\0') { }
 }
 
-void scrotNoteNew(char* format)
+void scrotNoteNew(char const *const format)
 {
+    char const *const end = format + strlen(format);
+
+    char *token = strpbrk(format, "-");
+
+    if (!token || (strlen(token) == 1))
+        errx(EXIT_FAILURE, "Error --note option : Malformed syntax.");
+
     scrotNoteFree();
 
-    note = (ScrotNote) {
-        NULL, NULL, 0, 0, 0.0,
-        { DEF_COLOR_RED, DEF_COLOR_GREEN,
-          DEF_COLOR_BLUE, DEF_COLOR_ALPHA }
-    };
+    note = calloc(1, sizeof(*note));
 
-    char* const end = format + strlen(format);
+    assert(note);
 
-    char* token = strpbrk(format, "-");
-
-    if (!token || (strlen(token) == 1)) {
-        malformed:
-
-            pfree(&format);
-            errx(EXIT_FAILURE, "Error --note option : Malformed syntax.");
-    }
+    note->color.r = DEF_COLOR_RED;
+    note->color.g = DEF_COLOR_GREEN;
+    note->color.b = DEF_COLOR_BLUE;
+    note->color.a = DEF_COLOR_ALPHA;
 
     while (token) {
         const char type = *++token;
-        char* savePtr = NULL;
-        char* c;
+        char *savePtr = NULL;
+        char *c;
+        const char *errmsg;
 
         nextSpace(&token);
 
         switch (type) {
         case 'f':
-            note.font = parseText(&token, end);
+            note->font = parseText(&token, end);
 
-            if (!note.font)
+            if (!note->font)
                 errx(EXIT_FAILURE, "Error --note option : Malformed syntax for -f");
 
-            char* number = strrchr(note.font, '/');
+            char *number = strrchr(note->font, '/');
 
             if (!number)
                 errx(EXIT_FAILURE, "Error --note option : Malformed syntax for -f, required number.");
 
-            int fontSize = optionsParseRequiredNumber(++number);
+            const int fontSize = optionsParseNum(++number, 1, INT_MAX,
+                &errmsg);
+            if (errmsg) {
+                    errx(EXIT_FAILURE, "option --note: font size '%s' is %s",
+                        number, errmsg);
+            }
 
             if (fontSize < 6)
                 warnx("Warning: --note option: font size < 6");
             break;
         case 'x':
-            if ((1 != sscanf(token, "%d", &note.x) || (note.x < 0)))
+            if ((1 != sscanf(token, "%d", &note->x) || (note->x < 0)))
                 errx(EXIT_FAILURE, "Error --note option : Malformed syntax for -x");
             nextNotSpace(&token);
             break;
         case 'y':
-            if ((1 != sscanf(token, "%d", &note.y)) || (note.y < 0))
+            if ((1 != sscanf(token, "%d", &note->y)) || (note->y < 0))
                 errx(EXIT_FAILURE, "Error --note option : Malformed syntax for -y");
             nextNotSpace(&token);
             break;
         case 'a':
-            if ((1 != sscanf(token, "%lf", &note.angle)))
+            if ((1 != sscanf(token, "%lf", &note->angle)))
                 errx(EXIT_FAILURE, "Error --note option : Malformed syntax for -a");
             nextNotSpace(&token);
             break;
         case 't':
-            note.text = parseText(&token, end);
-            if (!note.text)
+            note->text = parseText(&token, end);
+            if (!note->text)
                 errx(EXIT_FAILURE, "Error --note option : Malformed syntax for -t");
             break;
         case 'c':
@@ -136,22 +168,30 @@ void scrotNoteNew(char* format)
 
             while (c) {
                 token = c;
+                char *const space = strchr(c, ' ');
 
-                int const color = optionsParseRequireRange(
-                        optionsParseRequiredNumber(c), 0 ,255);
+                if (space)
+                    *space = '\0';
+                const int color = optionsParseNum(c, 0, 255, &errmsg);
+                if (errmsg) {
+                        errx(EXIT_FAILURE, "option --note: color '%s' is %s", c,
+                            errmsg);
+                }
+                if (space)
+                    *space = ' ';
 
                 switch (++numberColors) {
                 case 1:
-                    note.color.r = color;
+                    note->color.r = color;
                     break;
                 case 2:
-                    note.color.g = color;
+                    note->color.g = color;
                     break;
                 case 3:
-                    note.color.b = color;
+                    note->color.b = color;
                     break;
                 case 4:
-                    note.color.a = color;
+                    note->color.a = color;
                     break;
                 }
 
@@ -168,24 +208,30 @@ void scrotNoteNew(char* format)
         token = strpbrk(token, "-");
     }
 
-    if (!note.font || !note.text)
-        goto malformed;
+    if (!note->font || !note->text)
+        errx(EXIT_FAILURE, "Error --note option : Malformed syntax.");
 
     loadFont();
 }
 
 void scrotNoteFree(void)
 {
-    if (note.text)
-        pfree(&note.text);
+    if(!note)
+        return;
 
-    if (note.font)
-        pfree(&note.font);
+    if (note->text)
+        pfree(&note->text);
 
-    if (imFont) {
-        imlib_context_set_font(imFont);
+    if (note->font)
+        pfree(&note->font);
+
+    if (note->imFont) {
+        imlib_context_set_font(note->imFont);
         imlib_free_font();
     }
+
+    free(note);
+    note = NULL;
 }
 
 void scrotNoteDraw(Imlib_Image im)
@@ -193,32 +239,34 @@ void scrotNoteDraw(Imlib_Image im)
     if (!im)
         return;
 
+    assert(note);
+    assert(note->imFont);
+
     imlib_context_set_image(im);
-    imlib_context_set_font(imFont);
+    imlib_context_set_font(note->imFont);
 
     imlib_context_set_direction(IMLIB_TEXT_TO_ANGLE);
-    imlib_context_set_angle(note.angle);
+    imlib_context_set_angle(note->angle);
 
-    imlib_context_set_color(note.color.r,
-        note.color.g,
-        note.color.b,
-        note.color.a);
+    imlib_context_set_color(note->color.r,
+        note->color.g,
+        note->color.b,
+        note->color.a);
 
-    imlib_text_draw(note.x, note.y, note.text);
+    imlib_text_draw(note->x, note->y, note->text);
 }
 
-void loadFont(void)
+static void loadFont(void)
 {
-    imFont = imlib_load_font(note.font);
+    assert(note);
 
-    if (!imFont) {
-        warnx("Error --note option : Failed to load fontname: %s", note.font);
-        scrotNoteFree();
-        exit(EXIT_FAILURE);
-    }
+    note->imFont = imlib_load_font(note->font);
+
+    if (!note->imFont)
+        errx(EXIT_FAILURE, "Error --note option : Failed to load fontname: %s", note->font);
 }
 
-char* parseText(char** token, char* const end)
+static char *parseText(char **token, char const *const end)
 {
     assert(NULL != *token);
     assert(NULL != end);
@@ -228,7 +276,7 @@ char* parseText(char** token, char* const end)
 
     (*token)++;
 
-    char* begin = *token;
+    char *begin = *token;
 
     while ((*token != end) && **token != '\'')
         (*token)++;
