@@ -9,14 +9,15 @@ Copyright 2009      James Cameron <quozl@us.netrek.org>
 Copyright 2010      Ibragimov Rinat <ibragimovrinat@mail.ru>
 Copyright 2017      Stoney Sauce <stoneysauce@gmail.com>
 Copyright 2019      Daniel Lublin <daniel@lublin.se>
-Copyright 2019-2021 Daniel T. Borelli <danieltborelli@gmail.com>
+Copyright 2019-2022 Daniel T. Borelli <danieltborelli@gmail.com>
 Copyright 2019      Jade Auer <jade@trashwitch.dev>
 Copyright 2020      Sean Brennan <zettix1@gmail.com>
 Copyright 2021      Christopher R. Nelson <christopher.nelson@languidnights.com>
-Copyright 2021      Guilherme Janczak <guilherme.janczak@yandex.com>
+Copyright 2021-2023 Guilherme Janczak <guilherme.janczak@yandex.com>
 Copyright 2021      IFo Hancroft <contact@ifohancroft.com>
 Copyright 2021      Peter Wu <peterwu@hotmail.com>
 Copyright 2021      Wilson Smith <01wsmith+gh@gmail.com>
+Copyright 2022      Zev Weiss <zev@bewilderbeest.net>
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to
@@ -39,9 +40,23 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 */
 
+#include <assert.h>
+#include <err.h>
+#include <errno.h>
+#include <getopt.h>
+#include <limits.h>
+#include <stdbool.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
+#include <unistd.h>
+
+#include "config.h"
+#include "note.h"
 #include "options.h"
 #include "scrot.h"
-#include <assert.h>
+#include "scrot_selection.h"
+#include "util.h"
 
 #define STR_LEN_MAX_FILENAME(msg, fileName) do {                \
     if (strlen((fileName)) > MAX_FILENAME) {                    \
@@ -62,65 +77,84 @@ enum {
     MAX_DISPLAY_NAME = 256, // characters
 };
 
-ScrotOptions opt = {
+struct ScrotOptions opt = {
     .quality = 75,
     .lineStyle = LineSolid,
     .lineWidth = 1,
     .lineOpacity = SELECTION_OPACITY_DEFAULT,
     .lineMode = LINE_MODE_S_CLASSIC,
     .stackDirection = HORIZONTAL,
+    .monitor = -1,
 };
 
-int optionsParseRequiredNumber(char const* str)
+static void showUsage(void);
+static void showVersion(void);
+static void optionsParseThumbnail(char *);
+
+/* optionsParseNum: "string to number" function.
+ *
+ * Parses the string representation of an integer in str, and simultaneously
+ * ensures that it is >= min and <= max.
+ *
+ * Returns the integer and sets *errmsg to NULL on success.
+ * Returns 0 and sets *errmsg to a pointer to a string containing the
+ * reason why the number can't be parsed on error.
+ *
+ * usage:
+ * char *errmsg;
+ * unsigned int nonnegative;
+ * if ((nonnegative = optionsParseNum(optarg, 0, UINT_MAX, &errmsg)) == NULL)
+ *     errx(EXIT_FAILURE, "-n: '%s' is %s", optarg, errmsg);
+ */
+long long optionsParseNum(const char *str, long long min, long long max,
+    const char *errmsg[static 1])
 {
-    assert(NULL != str); // fix yout caller function,
-                         //  the user does not impose this behavior
-    char* end = NULL;
-    long ret = 0L;
-    errno = 0;
-
-    ret = strtol(str, &end, 10);
-
-    if (errno)
-        goto range_error;
-
-    if (str == end)
-        errx(EXIT_FAILURE, "the option is not a number: %s", end);
-
-    if (ret > INT_MAX || ret < INT_MIN) {
-        errno = ERANGE;
-        goto range_error;
+    char *end = NULL;
+    long long rval;
+    int saved_errno = errno;
+    if (str == NULL) {
+        *errmsg = "missing";
+        return 0;
     }
+    *errmsg = NULL;
 
-    return ret;
+    errno = 0;
+    rval = strtoll(str, &end, 10);
+    if (errno == ERANGE) {
+        *errmsg = "not representable";
+    } else if (*str == '\0') {
+        *errmsg = "the null string";
+    } else if (*end != '\0') {
+        *errmsg = "not a number";
+    } else if (rval < min) {
+        /*
+         * rval could be set to 0 due to strtoll() returning error and this
+         * could be smaller than min or larger than max. To make sure we don't
+         * return the wrong error message, put min/max checks after everything
+         * else.
+         */
+        *errmsg = min == 0 ? "negative" : "too small";
+    } else if (rval > max) {
+        *errmsg = "too large";
+    }
+    errno = saved_errno;
 
-range_error:
-    err(EXIT_FAILURE, "error strtol");
+    return (*errmsg ? 0 : rval);
 }
 
-static int nonNegativeNumber(int number)
-{
-    return (number < 0) ? 0 : number;
-}
-
-int optionsParseRequireRange(int n, int lo, int hi)
-{
-   return (n < lo ? lo : n > hi ? hi : n);
-}
-
-bool optionsParseIsString(char const* const str)
+static bool optionsParseIsString(const char *const str)
 {
     return (str && (str[0] != '\0'));
 }
 
-static void optionsParseStack(char const* optarg)
+static void optionsParseStack(const char *optarg)
 {
     // the suboption it's optional
     if (!optarg) {
         opt.stackDirection = HORIZONTAL;
         return;
     }
-    char const* value = strchr(optarg, '=');
+    const char *value = strchr(optarg, '=');
 
     if (value)
         ++value;
@@ -137,7 +171,7 @@ static void optionsParseStack(char const* optarg)
     }
 }
 
-static void optionsParseSelection(char const* optarg)
+static void optionsParseSelection(const char *optarg)
 {
     // the suboption it's optional
     if (!optarg) {
@@ -145,7 +179,7 @@ static void optionsParseSelection(char const* optarg)
         return;
     }
 
-    char const* value = strchr(optarg, '=');
+    const char *value = strchr(optarg, '=');
 
     if (value)
         ++value;
@@ -183,20 +217,20 @@ static void optionsParseSelection(char const* optarg)
         errx(EXIT_FAILURE, "option --select: Invalid parameter.");
 
     if (opt.selection.mode == SELECTION_MODE_BLUR) {
-        int const num = nonNegativeNumber(optionsParseRequiredNumber(value));
-
-        opt.selection.paramNum = optionsParseRequireRange(num,
-            SELECTION_MODE_BLUR_MIN, SELECTION_MODE_BLUR_MAX);
-
+        const char *errmsg;
+        opt.selection.paramNum = optionsParseNum(value,
+            SELECTION_MODE_BLUR_MIN, SELECTION_MODE_BLUR_MAX, &errmsg);
+        if (errmsg)
+            errx(EXIT_FAILURE, "option --select: '%s' is %s", value, errmsg);
     } else { // SELECTION_MODE_HIDE
 
         checkMaxInputFileName(value);
 
-        opt.selection.paramStr = strdup(value);
+        opt.selection.paramStr = estrdup(value);
     }
 }
 
-static void optionsParseLine(char* optarg)
+static void optionsParseLine(char *optarg)
 {
     enum {
         Style = 0,
@@ -206,7 +240,7 @@ static void optionsParseLine(char* optarg)
         Mode
     };
 
-    char* const token[] = {
+    char *const token[] = {
         [Style] = "style",
         [Width] = "width",
         [Color] = "color",
@@ -215,8 +249,9 @@ static void optionsParseLine(char* optarg)
         NULL
     };
 
-    char* subopts = optarg;
-    char* value = NULL;
+    char *subopts = optarg;
+    char *value = NULL;
+    const char *errmsg;
 
     while (*subopts != '\0') {
         switch (getsubopt(&subopts, token, &value)) {
@@ -236,16 +271,12 @@ static void optionsParseLine(char* optarg)
             }
             break;
         case Width:
-            if (!optionsParseIsString(value)) {
-                errx(EXIT_FAILURE, "Missing value for suboption '%s'",
-                    token[Width]);
-            }
-
-            opt.lineWidth = optionsParseRequiredNumber(value);
-
-            if (opt.lineWidth <= 0 || opt.lineWidth > 8) {
-                errx(EXIT_FAILURE, "Value of the range (1..8) for "
-                    "suboption '%s': %d", token[Width], opt.lineWidth);
+            opt.lineWidth = optionsParseNum(value, 1, 8, &errmsg);
+            if (errmsg) {
+                if (value == NULL)
+                    value = "(null)";
+                errx(EXIT_FAILURE, "option --line: suboption '%s': '%s' is %s",
+                    token[Width], value, errmsg);
             }
             break;
         case Color:
@@ -254,7 +285,7 @@ static void optionsParseLine(char* optarg)
                     token[Color]);
             }
 
-            opt.lineColor = strdup(value);
+            opt.lineColor = estrdup(value);
             break;
         case Mode:
             if (!optionsParseIsString(value)) {
@@ -271,14 +302,17 @@ static void optionsParseLine(char* optarg)
                     token[Mode], value);
             }
 
-            opt.lineMode = strdup(value);
+            opt.lineMode = estrdup(value);
             break;
         case Opacity:
-            if (!optionsParseIsString(value)) {
-                errx(EXIT_FAILURE, "Missing value for suboption '%s'",
-                    token[Opacity]);
+            opt.lineOpacity = optionsParseNum(value,
+                SELECTION_OPACITY_MIN, SELECTION_OPACITY_MAX, &errmsg);
+            if (errmsg) {
+                if (value == NULL)
+                        value = "(null)";
+                errx(EXIT_FAILURE, "option --line: suboption %s: '%s' is %s",
+                    token[Opacity], value, errmsg);
             }
-            opt.lineOpacity = optionsParseRequiredNumber(value);
             break;
         default:
             errx(EXIT_FAILURE, "No match found for token: '%s'", value);
@@ -287,7 +321,7 @@ static void optionsParseLine(char* optarg)
     } /* while */
 }
 
-static void optionsParseWindowClassName(const char* windowClassName)
+static void optionsParseWindowClassName(const char *windowClassName)
 {
     assert(windowClassName != NULL);
 
@@ -295,17 +329,16 @@ static void optionsParseWindowClassName(const char* windowClassName)
         opt.windowClassName = strndup(windowClassName, MAX_LEN_WINDOW_CLASS_NAME);
 }
 
-static bool accessFileOk(const char* const pathName)
+static bool accessFileOk(const char *const pathName)
 {
     errno = 0;
     return (0 == access(pathName, W_OK));
 }
 
-
-static char* getPathOfStdout(void)
+static char *getPathOfStdout(void)
 {
     char path[16] = {"/dev/stdout"};
-    size_t const len = sizeof(path);
+    const size_t len = sizeof(path);
 
     if (!accessFileOk(path)) {
 
@@ -324,9 +357,9 @@ static char* getPathOfStdout(void)
     return strndup(path, len);
 }
 
-void optionsParse(int argc, char** argv)
+void optionsParse(int argc, char *argv[])
 {
-    static char stropts[] = "a:ofipbcd:e:hmq:s::t:uvzn:l:D:k::C:S:F:";
+    static char stropts[] = "a:ofipbcd:e:hmq:s::t:uvzn:l:D:k::C:S:F:M:";
 
     static struct option lopts[] = {
         /* actions */
@@ -356,9 +389,11 @@ void optionsParse(int argc, char** argv)
         { "class", required_argument, 0, 'C' },
         { "script", required_argument, 0, 'S' },
         { "file", required_argument, 0, 'F' },
+        { "monitor", required_argument, 0, 'M'},
         { 0, 0, 0, 0 }
     };
     int optch = 0, cmdx = 0;
+    const char *errmsg;
 
     /* Now to pass some optionarinos */
     while ((optch = getopt_long(argc, argv, stropts, lopts, &cmdx)) != EOF) {
@@ -375,16 +410,24 @@ void optionsParse(int argc, char** argv)
             opt.border = 1;
             break;
         case 'd':
-            opt.delay = nonNegativeNumber(optionsParseRequiredNumber(optarg));
+            opt.delay = optionsParseNum(optarg, 0, INT_MAX, &errmsg);
+            if (errmsg) {
+                errx(EXIT_FAILURE, "option --delay: '%s' is %s", optarg,
+                    errmsg);
+            }
             break;
         case 'e':
-            opt.exec = strdup(optarg);
+            opt.exec = estrdup(optarg);
             break;
         case 'm':
             opt.multidisp = 1;
             break;
         case 'q':
-            opt.quality = optionsParseRequiredNumber(optarg);
+            opt.quality = optionsParseNum(optarg, 1, 100, &errmsg);
+            if (errmsg) {
+                errx(EXIT_FAILURE, "option --quality: '%s' is %s", optarg,
+                    errmsg);
+            }
             break;
         case 's':
             optionsParseSelection(optarg);
@@ -433,11 +476,18 @@ void optionsParse(int argc, char** argv)
             optionsParseWindowClassName(optarg);
             break;
         case 'S':
-            opt.script = strdup(optarg);
+            opt.script = estrdup(optarg);
             break;
         case 'F':
             optionsParseFileName(optarg);
             break;
+        case 'M':
+            opt.monitor = optionsParseNum(optarg, 0, INT_MAX, &errmsg);
+            if (errmsg) {
+                errx(EXIT_FAILURE, "option --monitor: '%s' is %s", optarg,
+                    errmsg);
+            }
+            break;
         case '?':
             exit(EXIT_FAILURE);
         default:
@@ -452,13 +502,13 @@ void optionsParse(int argc, char** argv)
         if (!opt.outputFile) {
             optionsParseFileName(argv[optind++]);
 
-            bool const redirectChar = ( opt.outputFile[0] == '-'
+            const bool redirectChar = ( opt.outputFile[0] == '-'
                                         && opt.outputFile[1] == '\0');
             if (redirectChar) {
                 free(opt.outputFile);
                 opt.outputFile = getPathOfStdout();
                 opt.overwrite = 1;
-                opt.thumb = 0;
+                opt.thumbWorP = 0;
             }
         } else
             warnx("unrecognised option %s", argv[optind++]);
@@ -468,17 +518,33 @@ void optionsParse(int argc, char** argv)
     optind = 1;
 }
 
-char* optionsNameThumbnail(const char* name)
+static void showUsage(void)
 {
-    const char* const thumbSuffix = "-thumb";
+    fputs(/* Check that everything lines up after any changes. */
+        "usage:  " PACKAGE " [-bcfhimopuvz] [-a X,Y,W,H] [-C NAME] [-D DISPLAY]\n"
+        "              [-d SEC] [-e CMD] [-F FILE] [-k OPT] [-l STYLE] [-M NUM]\n"
+        "              [-n OPTS] [-q NUM] [-S CMD] [-s OPTS] [-t NUM | GEOM] [FILE]\n",
+        stdout);
+    exit(0);
+}
+
+static void showVersion(void)
+{
+    printf(PACKAGE " version " VERSION "\n");
+    exit(0);
+}
+
+char *optionsNameThumbnail(const char *name)
+{
+    const char *const thumbSuffix = "-thumb";
     const size_t thumbSuffixLength = 7;
     const size_t newNameLength = strlen(name) + thumbSuffixLength;
-    char* newName = calloc(1, newNameLength);
+    char *newName = calloc(1, newNameLength);
 
     if (!newName)
         err(EXIT_FAILURE, "Unable to allocate thumbnail");
 
-    const char* const extension = strrchr(name, '.');
+    const char *const extension = strrchr(name, '.');
 
     if (extension) {
         /* We add one so length includes '\0'*/
@@ -492,76 +558,83 @@ char* optionsNameThumbnail(const char* name)
     return newName;
 }
 
-void optionsParseAutoselect(char* optarg)
+void optionsParseAutoselect(char *optarg)
 {
-    char* token;
-    const char tokenDelimiter[2] = ",";
-    int dimensions[4];
+    char *token;
+    int *dimensions[] = {&opt.autoselectX, &opt.autoselectY, &opt.autoselectW,
+        &opt.autoselectH, NULL /* Sentinel. */};
     int i = 0;
+    int min;
+    const char *errmsg;
+
+    /* Geometry dimensions must be in format x,y,w,h */
+    token = strtok(optarg, ",");
+    for (; token != NULL; token = strtok(NULL, ",")) {
+        if (dimensions[i] == NULL)
+            errx(EXIT_FAILURE, "option --autoselect: too many dimensions");
+
+        min = i >= 2; /* X,Y offsets may be 0. Width and height may not. */
+        *dimensions[i] = optionsParseNum(token, min, INT_MAX, &errmsg);
+        if (errmsg) {
+            errx(EXIT_FAILURE, "option --autoselect: '%s' is %s", token,
+                errmsg);
+        }
+        i++;
 
-    if (strchr(optarg, ',')) { /* geometry dimensions must be in format x,y,w,h   */
-        dimensions[i++] = optionsParseRequiredNumber(strtok(optarg, tokenDelimiter));
-        while ((token = strtok(NULL, tokenDelimiter)))
-            dimensions[i++] = optionsParseRequiredNumber(token);
         opt.autoselect = 1;
-        opt.autoselectX = dimensions[0];
-        opt.autoselectY = dimensions[1];
-        opt.autoselectW = dimensions[2];
-        opt.autoselectH = dimensions[3];
-
-        if (i != 4)
-            errx(EXIT_FAILURE, "option 'autoselect' require 4 arguments");
-    } else
-        errx(EXIT_FAILURE, "invalid format for option -- 'autoselect'");
+    }
+    if (i < 4)
+            errx(EXIT_FAILURE, "option --autoselect: too few dimensions");
 }
 
-void optionsParseDisplay(char* optarg)
+void optionsParseDisplay(char *optarg)
 {
     opt.display = strndup(optarg, MAX_DISPLAY_NAME);
     if (!opt.display)
         err(EXIT_FAILURE, "Unable to allocate display");
 }
 
-void optionsParseThumbnail(char* optarg)
+static void optionsParseThumbnail(char *optarg)
 {
-    char* token;
+    char *height;
+    const char *errmsg;
 
-    if (strchr(optarg, 'x')) { /* We want to specify the geometry */
-        token = strtok(optarg, "x");
-        opt.thumbWidth = optionsParseRequiredNumber(token);
-        token = strtok(NULL, "x");
-        if (token) {
-            opt.thumbWidth = optionsParseRequiredNumber(optarg);
-            opt.thumbHeight = optionsParseRequiredNumber(token);
+    if ((height = strchr(optarg, 'x')) != NULL) { /* optarg is a resolution. */
+        /* optarg holds the width, height holds the height. */
+        *height++ = '\0';
 
-            if (opt.thumbWidth < 0)
-                opt.thumbWidth = 1;
-            if (opt.thumbHeight < 0)
-                opt.thumbHeight = 1;
+        opt.thumbWorP = optionsParseNum(optarg, 1, INT_MAX, &errmsg);
+        if (errmsg) {
+            errx(EXIT_FAILURE, "option --thumb: resolution width '%s' is %s",
+                optarg, errmsg);
+        }
 
-            if (!opt.thumbWidth && !opt.thumbHeight)
-                opt.thumb = 0;
-            else
-                opt.thumb = 1;
+        opt.thumbH = optionsParseNum(height, 1, INT_MAX, &errmsg);
+        if (errmsg) {
+            errx(EXIT_FAILURE, "option --thumb: resolution height '%s' is %s",
+                height, errmsg);
+        }
+    } else { /* optarg is a percentage. */
+        opt.thumbWorP = optionsParseNum(optarg, 1, INT_MAX, &errmsg);
+        if (errmsg) {
+            errx(EXIT_FAILURE, "option --thumb: percentage '%s' is %s", optarg,
+                errmsg);
         }
-    } else {
-        opt.thumb = optionsParseRequiredNumber(optarg);
-        if (opt.thumb < 1)
-            opt.thumb = 1;
-        else if (opt.thumb > 100)
-            opt.thumb = 100;
     }
 }
 
-void optionsParseFileName(const char* optarg)
+void optionsParseFileName(const char *optarg)
 {
     checkMaxOutputFileName(optarg);
-    opt.outputFile = strdup(optarg);
+    opt.outputFile = estrdup(optarg);
 }
 
-void optionsParseNote(char* optarg)
+void optionsParseNote(char *optarg)
 {
-    opt.note = strdup(optarg);
+    if (opt.note)
+        free(opt.note);
+
+    opt.note = estrdup(optarg);
 
     if (!opt.note)
         return;
@@ -577,26 +650,9 @@ Return:
     0 : It does not match
     1 : If it matches
 */
-int optionsCompareWindowClassName(const char* targetClassName)
+int optionsCompareWindowClassName(const char *targetClassName)
 {
     assert(targetClassName != NULL);
     assert(opt.windowClassName != NULL);
     return !!(!strncmp(targetClassName, opt.windowClassName, MAX_LEN_WINDOW_CLASS_NAME - 1));
 }
-
-void showVersion(void)
-{
-    printf(SCROT_PACKAGE " version " SCROT_VERSION "\n");
-    exit(0);
-}
-
-void showUsage(void)
-{
-    fputs(/* Check that everything lines up after any changes. */
-        "usage:  " SCROT_PACKAGE " [-bcfhikmopsuvz] [-a X,Y,W,H] [-C NAME] [-D DISPLAY]"
-        "\n"
-        "              [-F FILE] [-d SEC] [-e CMD] [-l STYLE] [-n OPTS] [-q NUM] [-S CMD] \n"
-        "              [-t NUM | GEOM] [FILE]\n",
-        stdout);
-    exit(0);
-}
