@@ -6,11 +6,13 @@ Copyright 1999-2000 Tom Gilbert <tom@linuxbrit.co.uk,
 Copyright 2009      James Cameron <quozl@us.netrek.org>
 Copyright 2010      Ibragimov Rinat <ibragimovrinat@mail.ru>
 Copyright 2017      Stoney Sauce <stoneysauce@gmail.com>
-Copyright 2019-2021 Daniel T. Borelli <danieltborelli@gmail.com>
+Copyright 2019-2022 Daniel T. Borelli <danieltborelli@gmail.com>
 Copyright 2020      Sean Brennan <zettix1@gmail.com>
+Copyright 2021-2023 Guilherme Janczak <guilherme.janczak@yandex.com>
 Copyright 2021      IFo Hancroft <contact@ifohancroft.com>
 Copyright 2021      Peter Wu <peterwu@hotmail.com>
 Copyright 2021      Wilson Smith <01wsmith+gh@gmail.com>
+Copyright 2022      Zev Weiss <zev@bewilderbeest.net>
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to
@@ -33,19 +35,18 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 */
 
-#include "structs.h"
-#include "scrot_selection.h"
-
 #pragma once
 
+#include "scrot_selection.h"
+
 // General purpose enum
 enum Direction {
-    // see main.c:stalkImageConcat(...)
+    // see scrot.c:stalkImageConcat(...)
     HORIZONTAL,
     VERTICAL,
 };
 
-struct __ScrotOptions {
+struct ScrotOptions {
     int delay;
     int countdown;
     int focused;
@@ -53,9 +54,12 @@ struct __ScrotOptions {
     int border;
     int silent;
     int multidisp;
-    int thumb;
-    int thumbWidth;
-    int thumbHeight;
+    /* If true, the --thumb option is in use. */
+    int thumbWorP;
+    /* If thumbH is true, thumbWorP contains a width and thumbH contains a
+     * height. Otherwise, thumbWorP contains a percentage.
+     */
+    int thumbH;
     int pointer;
     int ignoreKeyboard;
     int freeze;
@@ -65,31 +69,32 @@ struct __ScrotOptions {
     int lineOpacity;
     int stack;
     enum Direction stackDirection;
-    char* lineColor;
-    char* lineMode;
-    char* outputFile;
-    char* thumbFile;
-    char* exec;
-    char* display;
-    char* note;
-    char* windowClassName;
-    char* script;
+    char *lineColor;
+    char *lineMode;
+    char *outputFile;
+    char *thumbFile;
+    char *exec;
+    char *display;
+    char *note;
+    char *windowClassName;
+    char *script;
     int autoselect;
     int autoselectX;
     int autoselectY;
     int autoselectH;
     int autoselectW;
     SelectionMode selection;
+    int monitor;
 };
 
-void optionsParse(int, char**);
-char* optionsNameThumbnail(const char*);
-void optionsParseFileName(const char*);
-void optionsParseThumbnail(char*);
-void optionsParseAutoselect(char*);
-void optionsParseDisplay(char*);
-void optionsParseNote(char*);
-int optionsParseRequiredNumber(char const *);
-int optionsCompareWindowClassName(const char*);
-int optionsParseRequireRange(int, int, int);
-extern ScrotOptions opt;
+extern struct ScrotOptions opt;
+
+void optionsParse(int, char **);
+char *optionsNameThumbnail(const char *);
+void optionsParseFileName(const char *);
+void optionsParseAutoselect(char *);
+void optionsParseDisplay(char *);
+void optionsParseNote(char *);
+long long optionsParseNum(const char *, long long, long long,
+    const char *[static 1]);
+int optionsCompareWindowClassName(const char *);
